@@ -29,11 +29,11 @@
  *     POSSIBILITY OF SUCH DAMAGE.
  *
  *
- * \file	encoder.c
+ * \file    encoder.c
  *
- * \brief	core encoder
+ * \brief   core encoder
  *
- * \date	5/14/2009 Created
+ * \date    5/14/2009 Created
  *
  *************************************************************************************
  */
@@ -46,12 +46,14 @@
 #include "deblocking.h"
 #include "ref_list_mgr_svc.h"
 #include "mc.h"
+#include "paraset_strategy.h"
 #include "sample.h"
 
+#include "svc_enc_golomb.h"
 #include "svc_base_layer_md.h"
 #include "svc_mode_decision.h"
 #include "set_mb_syn_cavlc.h"
-#include "crt_util_safe_x.h"	// Safe CRT routines like utils for cross_platforms
+#include "crt_util_safe_x.h" // Safe CRT routines like utils for cross_platforms
 #include "slice_multi_threading.h"
 
 //  global   function  pointers  definition
@@ -60,12 +62,12 @@ namespace WelsEnc {
 
 
 /*!
- * \brief	initialize source picture body
- * \param	pSrc		SSourcePicture*
- * \param	csp		internal csp format
- * \param	iWidth	widht of picture in pixels
- * \param	iHeight	iHeight of picture in pixels
- * \return	successful - 0; otherwise none 0 for failed
+ * \brief   initialize source picture body
+ * \param   pSrc        SSourcePicture*
+ * \param   csp         internal csp format
+ * \param   iWidth      widht of picture in pixels
+ * \param   iHeight     iHeight of picture in pixels
+ * \return  successful - 0; otherwise none 0 for failed
  */
 int32_t InitPic (const void* kpSrc, const int32_t kiColorspace, const int32_t kiWidth, const int32_t kiHeight) {
   SSourcePicture* pSrcPic = (SSourcePicture*)kpSrc;
@@ -73,9 +75,9 @@ int32_t InitPic (const void* kpSrc, const int32_t kiColorspace, const int32_t ki
   if (NULL == pSrcPic || kiWidth == 0 || kiHeight == 0)
     return 1;
 
-  pSrcPic->iColorFormat	= kiColorspace;
-  pSrcPic->iPicWidth		= kiWidth;
-  pSrcPic->iPicHeight		= kiHeight;
+  pSrcPic->iColorFormat = kiColorspace;
+  pSrcPic->iPicWidth    = kiWidth;
+  pSrcPic->iPicHeight   = kiHeight;
 
   //currently encoder only supports videoFormatI420.
   if ((kiColorspace & (~videoFormatVFlip)) != videoFormatI420)
@@ -83,32 +85,32 @@ int32_t InitPic (const void* kpSrc, const int32_t kiColorspace, const int32_t ki
   switch (kiColorspace & (~videoFormatVFlip)) {
   case videoFormatI420:
   case videoFormatYV12:
-    pSrcPic->pData[0]	= NULL;
-    pSrcPic->pData[1]	= NULL;
-    pSrcPic->pData[2]	= NULL;
-    pSrcPic->pData[3]	= NULL;
-    pSrcPic->iStride[0]	= kiWidth;
-    pSrcPic->iStride[2]	= pSrcPic->iStride[1] = kiWidth >> 1;
-    pSrcPic->iStride[3]	= 0;
+    pSrcPic->pData[0]   = NULL;
+    pSrcPic->pData[1]   = NULL;
+    pSrcPic->pData[2]   = NULL;
+    pSrcPic->pData[3]   = NULL;
+    pSrcPic->iStride[0] = kiWidth;
+    pSrcPic->iStride[2] = pSrcPic->iStride[1] = kiWidth >> 1;
+    pSrcPic->iStride[3] = 0;
     break;
   case videoFormatYUY2:
   case videoFormatYVYU:
   case videoFormatUYVY:
-    pSrcPic->pData[0]	= NULL;
-    pSrcPic->pData[1]	= NULL;
-    pSrcPic->pData[2]	= NULL;
-    pSrcPic->pData[3]	= NULL;
-    pSrcPic->iStride[0]	= CALC_BI_STRIDE (kiWidth,  16);
-    pSrcPic->iStride[3]	= pSrcPic->iStride[2] = pSrcPic->iStride[1] = 0;
+    pSrcPic->pData[0]   = NULL;
+    pSrcPic->pData[1]   = NULL;
+    pSrcPic->pData[2]   = NULL;
+    pSrcPic->pData[3]   = NULL;
+    pSrcPic->iStride[0] = CALC_BI_STRIDE (kiWidth,  16);
+    pSrcPic->iStride[3] = pSrcPic->iStride[2] = pSrcPic->iStride[1] = 0;
     break;
   case videoFormatRGB:
   case videoFormatBGR:
-    pSrcPic->pData[0]	= NULL;
-    pSrcPic->pData[1]	= NULL;
-    pSrcPic->pData[2]	= NULL;
-    pSrcPic->pData[3]	= NULL;
-    pSrcPic->iStride[0]	= CALC_BI_STRIDE (kiWidth, 24);
-    pSrcPic->iStride[3]	= pSrcPic->iStride[2] = pSrcPic->iStride[1] = 0;
+    pSrcPic->pData[0]   = NULL;
+    pSrcPic->pData[1]   = NULL;
+    pSrcPic->pData[2]   = NULL;
+    pSrcPic->pData[3]   = NULL;
+    pSrcPic->iStride[0] = CALC_BI_STRIDE (kiWidth, 24);
+    pSrcPic->iStride[3] = pSrcPic->iStride[2] = pSrcPic->iStride[1] = 0;
     if (kiColorspace & videoFormatVFlip)
       pSrcPic->iColorFormat = kiColorspace & (~videoFormatVFlip);
     else
@@ -118,19 +120,19 @@ int32_t InitPic (const void* kpSrc, const int32_t kiColorspace, const int32_t ki
   case videoFormatRGBA:
   case videoFormatARGB:
   case videoFormatABGR:
-    pSrcPic->pData[0]	= NULL;
-    pSrcPic->pData[1]	= NULL;
-    pSrcPic->pData[2]	= NULL;
-    pSrcPic->pData[3]	= NULL;
-    pSrcPic->iStride[0]	= kiWidth << 2;
-    pSrcPic->iStride[3]	= pSrcPic->iStride[2] = pSrcPic->iStride[1] = 0;
+    pSrcPic->pData[0]   = NULL;
+    pSrcPic->pData[1]   = NULL;
+    pSrcPic->pData[2]   = NULL;
+    pSrcPic->pData[3]   = NULL;
+    pSrcPic->iStride[0] = kiWidth << 2;
+    pSrcPic->iStride[3] = pSrcPic->iStride[2] = pSrcPic->iStride[1] = 0;
     if (kiColorspace & videoFormatVFlip)
       pSrcPic->iColorFormat = kiColorspace & (~videoFormatVFlip);
     else
       pSrcPic->iColorFormat = kiColorspace | videoFormatVFlip;
     break;
   default:
-    return 2;	// any else?
+    return 2; // any else?
   }
 
   return 0;
@@ -140,17 +142,17 @@ int32_t InitPic (const void* kpSrc, const int32_t kiColorspace, const int32_t ki
 void WelsInitBGDFunc (SWelsFuncPtrList* pFuncList, const bool kbEnableBackgroundDetection) {
   if (kbEnableBackgroundDetection) {
     pFuncList->pfInterMdBackgroundDecision = WelsMdInterJudgeBGDPskip;
-    pFuncList->pfInterMdBackgroundInfoUpdate = WelsMdInterUpdateBGDInfo;
+    pFuncList->pfMdBackgroundInfoUpdate = WelsMdUpdateBGDInfo;
   } else {
     pFuncList->pfInterMdBackgroundDecision = WelsMdInterJudgeBGDPskipFalse;
-    pFuncList->pfInterMdBackgroundInfoUpdate = WelsMdInterUpdateBGDInfoNULL;
+    pFuncList->pfMdBackgroundInfoUpdate = WelsMdUpdateBGDInfoNULL;
   }
 }
 
 /*!
- * \brief	initialize function pointers that potentially used in Wels encoding
- * \param	pEncCtx		sWelsEncCtx*
- * \return	successful - 0; otherwise none 0 for failed
+ * \brief   initialize function pointers that potentially used in Wels encoding
+ * \param   pEncCtx     sWelsEncCtx*
+ * \return  successful - 0; otherwise none 0 for failed
  */
 int32_t InitFunctionPointers (sWelsEncCtx* pEncCtx, SWelsSvcCodingParam* pParam, uint32_t uiCpuFlag) {
   int32_t iReturn = ENC_RETURN_SUCCESS;
@@ -158,33 +160,33 @@ int32_t InitFunctionPointers (sWelsEncCtx* pEncCtx, SWelsSvcCodingParam* pParam,
   bool bScreenContent = (SCREEN_CONTENT_REAL_TIME == pParam->iUsageType);
 
   /* Functionality utilization of CPU instructions dependency */
-  pFuncList->pfSetMemZeroSize8	= WelsSetMemZero_c;		// confirmed_safe_unsafe_usage
-  pFuncList->pfSetMemZeroSize64Aligned16	= WelsSetMemZero_c;	// confirmed_safe_unsafe_usage
-  pFuncList->pfSetMemZeroSize64	= WelsSetMemZero_c;	// confirmed_safe_unsafe_usage
+  pFuncList->pfSetMemZeroSize8              = WelsSetMemZero_c;             // confirmed_safe_unsafe_usage
+  pFuncList->pfSetMemZeroSize64Aligned16    = WelsSetMemZero_c;     // confirmed_safe_unsafe_usage
+  pFuncList->pfSetMemZeroSize64             = WelsSetMemZero_c;     // confirmed_safe_unsafe_usage
 #if defined(X86_ASM)
   if (uiCpuFlag & WELS_CPU_MMXEXT) {
-    pFuncList->pfSetMemZeroSize8	= WelsSetMemZeroSize8_mmx;		// confirmed_safe_unsafe_usage
-    pFuncList->pfSetMemZeroSize64Aligned16	= WelsSetMemZeroSize64_mmx;	// confirmed_safe_unsafe_usage
-    pFuncList->pfSetMemZeroSize64	= WelsSetMemZeroSize64_mmx;	// confirmed_safe_unsafe_usage
+    pFuncList->pfSetMemZeroSize8            = WelsSetMemZeroSize8_mmx;              // confirmed_safe_unsafe_usage
+    pFuncList->pfSetMemZeroSize64Aligned16  = WelsSetMemZeroSize64_mmx;     // confirmed_safe_unsafe_usage
+    pFuncList->pfSetMemZeroSize64           = WelsSetMemZeroSize64_mmx;     // confirmed_safe_unsafe_usage
   }
   if (uiCpuFlag & WELS_CPU_SSE2) {
-    pFuncList->pfSetMemZeroSize64Aligned16	= WelsSetMemZeroAligned64_sse2;	// confirmed_safe_unsafe_usage
+    pFuncList->pfSetMemZeroSize64Aligned16  = WelsSetMemZeroAligned64_sse2; // confirmed_safe_unsafe_usage
   }
 #endif//X86_ASM
 
 #if defined(HAVE_NEON)
   if (uiCpuFlag & WELS_CPU_NEON) {
-    pFuncList->pfSetMemZeroSize8	= WelsSetMemZero_neon;
-    pFuncList->pfSetMemZeroSize64Aligned16	= WelsSetMemZero_neon;
-    pFuncList->pfSetMemZeroSize64	= WelsSetMemZero_neon;
+    pFuncList->pfSetMemZeroSize8            = WelsSetMemZero_neon;
+    pFuncList->pfSetMemZeroSize64Aligned16  = WelsSetMemZero_neon;
+    pFuncList->pfSetMemZeroSize64           = WelsSetMemZero_neon;
   }
 #endif
 
 #if defined(HAVE_NEON_AARCH64)
   if (uiCpuFlag & WELS_CPU_NEON) {
-    pFuncList->pfSetMemZeroSize8	= WelsSetMemZero_AArch64_neon;
-    pFuncList->pfSetMemZeroSize64Aligned16	= WelsSetMemZero_AArch64_neon;
-    pFuncList->pfSetMemZeroSize64	= WelsSetMemZero_AArch64_neon;
+    pFuncList->pfSetMemZeroSize8            = WelsSetMemZero_AArch64_neon;
+    pFuncList->pfSetMemZeroSize64Aligned16  = WelsSetMemZero_AArch64_neon;
+    pFuncList->pfSetMemZeroSize64           = WelsSetMemZero_AArch64_neon;
   }
 #endif
 
@@ -201,7 +203,9 @@ int32_t InitFunctionPointers (sWelsEncCtx* pEncCtx, SWelsSvcCodingParam* pParam,
 
   //
   WelsInitBGDFunc (pFuncList, pParam->bEnableBackgroundDetection);
-  WelsInitSCDPskipFunc (pFuncList, bScreenContent && (pParam->bEnableSceneChangeDetect));
+	WelsInitSCDPskipFunc (pFuncList, bScreenContent &&
+                        (pParam->bEnableSceneChangeDetect) &&
+                        (pEncCtx->pSvcParam->iComplexityMode < HIGH_COMPLEXITY));
 
   // for pfGetVarianceFromIntraVaa function ptr adaptive by CPU features, 6/7/2010
   InitIntraAnalysisVaaInfo (pFuncList, uiCpuFlag);
@@ -210,7 +214,7 @@ int32_t InitFunctionPointers (sWelsEncCtx* pEncCtx, SWelsSvcCodingParam* pParam,
   /*init pixel average function*/
   /*get one column or row pixel when refinement*/
   InitMcFunc (&pFuncList->sMcFuncs, uiCpuFlag);
-  InitCoeffFunc (pFuncList,uiCpuFlag,pParam->iEntropyCodingModeFlag);
+  InitCoeffFunc (pFuncList, uiCpuFlag, pParam->iEntropyCodingModeFlag);
 
   WelsInitEncodingFuncs (pFuncList, uiCpuFlag);
   WelsInitReconstructionFuncs (pFuncList, uiCpuFlag);
@@ -220,72 +224,106 @@ int32_t InitFunctionPointers (sWelsEncCtx* pEncCtx, SWelsSvcCodingParam* pParam,
 
   InitFillNeighborCacheInterFunc (pFuncList, pParam->bEnableBackgroundDetection);
 
-  InitRefListMgrFunc (pFuncList, pParam->bEnableLongTermReference, bScreenContent);
+  pFuncList->pParametersetStrategy = IWelsParametersetStrategy::CreateParametersetStrategy (pParam->eSpsPpsIdStrategy,
+                                     pParam->bSimulcastAVC, pParam->iSpatialLayerNum);
+  WELS_VERIFY_RETURN_IF (ENC_RETURN_MEMALLOCERR, (NULL == pFuncList->pParametersetStrategy))
 
   return iReturn;
 }
 
-/*!
- * \brief	initialize frame coding
- */
-void InitFrameCoding (sWelsEncCtx* pEncCtx, const EVideoFrameType keFrameType) {
+void UpdateFrameNum (sWelsEncCtx* pEncCtx, const int32_t kiDidx) {
+  SSpatialLayerInternal* pParamInternal = &pEncCtx->pSvcParam->sDependencyLayers[kiDidx];
+  bool bNeedFrameNumIncreasing = false;
+
+  if (NRI_PRI_LOWEST != pEncCtx->eLastNalPriority[kiDidx]) {
+    bNeedFrameNumIncreasing = true;
+  }
+
+  if (bNeedFrameNumIncreasing) {
+    if (pParamInternal->iFrameNum < (1 << pEncCtx->pSps->uiLog2MaxFrameNum) - 1)
+      ++ pParamInternal->iFrameNum;
+    else
+      pParamInternal->iFrameNum = 0;    // if iFrameNum overflow
+  }
+
+  pEncCtx->eLastNalPriority[kiDidx] = NRI_PRI_LOWEST;
+}
+
+
+void LoadBackFrameNum (sWelsEncCtx* pEncCtx, const int32_t kiDidx) {
+  SSpatialLayerInternal* pParamInternal = &pEncCtx->pSvcParam->sDependencyLayers[kiDidx];
+  bool bNeedFrameNumIncreasing = false;
+
+  if (NRI_PRI_LOWEST != pEncCtx->eLastNalPriority[kiDidx]) {
+    bNeedFrameNumIncreasing = true;
+  }
+
+  if (bNeedFrameNumIncreasing) {
+    if (pParamInternal->iFrameNum != 0) {
+      pParamInternal->iFrameNum --;
+    } else {
+      pParamInternal->iFrameNum = (1 << pEncCtx->pSps->uiLog2MaxFrameNum) - 1;
+    }
+  }
+}
+
+void InitBitStream (sWelsEncCtx* pEncCtx) {
   // for bitstream writing
-  pEncCtx->iPosBsBuffer		= 0;	// reset bs pBuffer position
-  pEncCtx->pOut->iNalIndex		= 0;	// reset NAL index
+  pEncCtx->iPosBsBuffer         = 0;    // reset bs pBuffer position
+  pEncCtx->pOut->iNalIndex      = 0;    // reset NAL index
+  pEncCtx->pOut->iLayerBsIndex  = 0;    // reset index of Layer Bs
 
   InitBits (&pEncCtx->pOut->sBsWrite, pEncCtx->pOut->pBsBuffer, pEncCtx->pOut->uiSize);
-
+}
+/*!
+ * \brief   initialize frame coding
+ */
+void InitFrameCoding (sWelsEncCtx* pEncCtx, const EVideoFrameType keFrameType, const int32_t kiDidx) {
+  SSpatialLayerInternal* pParamInternal = &pEncCtx->pSvcParam->sDependencyLayers[kiDidx];
   if (keFrameType == videoFrameTypeP) {
-    ++pEncCtx->iFrameIndex;
+    ++pParamInternal->iFrameIndex;
 
-    if (pEncCtx->iPOC < (1 << pEncCtx->pSps->iLog2MaxPocLsb) - 2)     // if iPOC type is no 0, this need be modification
-      pEncCtx->iPOC			+= 2;	// for POC type 0
+    if (pParamInternal->iPOC < (1 << pEncCtx->pSps->iLog2MaxPocLsb) -
+        2)     // if iPOC type is no 0, this need be modification
+      pParamInternal->iPOC += 2;   // for POC type 0
     else
-      pEncCtx->iPOC = 0;
+      pParamInternal->iPOC = 0;
 
-    if (pEncCtx->eLastNalPriority != 0) {
-      if (pEncCtx->iFrameNum < (1 << pEncCtx->pSps->uiLog2MaxFrameNum) - 1)
-        ++ pEncCtx->iFrameNum;
-      else
-        pEncCtx->iFrameNum	= 0;	// if iFrameNum overflow
-    }
-    pEncCtx->eNalType		= NAL_UNIT_CODED_SLICE;
-    pEncCtx->eSliceType	= P_SLICE;
-    pEncCtx->eNalPriority	= NRI_PRI_HIGH;
+    UpdateFrameNum (pEncCtx, kiDidx);
+
+    pEncCtx->eNalType           = NAL_UNIT_CODED_SLICE;
+    pEncCtx->eSliceType         = P_SLICE;
+    pEncCtx->eNalPriority       = NRI_PRI_HIGH;
   } else if (keFrameType == videoFrameTypeIDR) {
-    pEncCtx->iFrameNum		= 0;
-    pEncCtx->iPOC			= 0;
-    pEncCtx->bEncCurFrmAsIdrFlag = false;
-    pEncCtx->iFrameIndex = 0;
+    pParamInternal->iFrameNum          = 0;
+    pParamInternal->iPOC               = 0;
+    pParamInternal->bEncCurFrmAsIdrFlag = false;
+    pParamInternal->iFrameIndex = 0;
 
-    pEncCtx->eNalType		= NAL_UNIT_CODED_SLICE_IDR;
-    pEncCtx->eSliceType	= I_SLICE;
-    pEncCtx->eNalPriority	= NRI_PRI_HIGHEST;
+    pEncCtx->eNalType           = NAL_UNIT_CODED_SLICE_IDR;
+    pEncCtx->eSliceType         = I_SLICE;
+    pEncCtx->eNalPriority       = NRI_PRI_HIGHEST;
 
-    pEncCtx->iCodingIndex	= 0;
+    pParamInternal->iCodingIndex       = 0;
 
     // reset_ref_list
 
     // rc_init_gop
   } else if (keFrameType == videoFrameTypeI) {
-    if (pEncCtx->iPOC < (1 << pEncCtx->pSps->iLog2MaxPocLsb) - 2)     // if iPOC type is no 0, this need be modification
-      pEncCtx->iPOC			+= 2;	// for POC type 0
+    if (pParamInternal->iPOC < (1 << pEncCtx->pSps->iLog2MaxPocLsb) -
+        2)     // if iPOC type is no 0, this need be modification
+      pParamInternal->iPOC += 2;   // for POC type 0
     else
-      pEncCtx->iPOC = 0;
+      pParamInternal->iPOC = 0;
 
-    if (pEncCtx->eLastNalPriority != 0) {
-      if (pEncCtx->iFrameNum < (1 << pEncCtx->pSps->uiLog2MaxFrameNum) - 1)
-        ++ pEncCtx->iFrameNum;
-      else
-        pEncCtx->iFrameNum	= 0;	// if iFrameNum overflow
-    }
+    UpdateFrameNum (pEncCtx, kiDidx);
 
-    pEncCtx->eNalType		= NAL_UNIT_CODED_SLICE;
-    pEncCtx->eSliceType	= I_SLICE;
-    pEncCtx->eNalPriority	= NRI_PRI_HIGHEST;
+    pEncCtx->eNalType     = NAL_UNIT_CODED_SLICE;
+    pEncCtx->eSliceType   = I_SLICE;
+    pEncCtx->eNalPriority = NRI_PRI_HIGHEST;
 
     // rc_init_gop
-  } else {	// B pictures are not supported now, any else?
+  } else { // B pictures are not supported now, any else?
     assert (0);
   }
 
@@ -294,11 +332,12 @@ void InitFrameCoding (sWelsEncCtx* pEncCtx, const EVideoFrameType keFrameType) {
 #endif//FRAME_INFO_OUTPUT
 }
 
-EVideoFrameType DecideFrameType (sWelsEncCtx* pEncCtx, const int8_t kiSpatialNum) {
-  SWelsSvcCodingParam* pSvcParam	= pEncCtx->pSvcParam;
+EVideoFrameType DecideFrameType (sWelsEncCtx* pEncCtx, const int8_t kiSpatialNum, const int32_t kiDidx,
+                                 bool bSkipFrameFlag) {
+  SWelsSvcCodingParam* pSvcParam = pEncCtx->pSvcParam;
+  SSpatialLayerInternal* pParamInternal = &pEncCtx->pSvcParam->sDependencyLayers[kiDidx];
   EVideoFrameType iFrameType = videoFrameTypeInvalid;
   bool bSceneChangeFlag = false;
-
   if (pSvcParam->iUsageType == SCREEN_CONTENT_REAL_TIME) {
     if ((!pSvcParam->bEnableSceneChangeDetect) || pEncCtx->pVaa->bIdrPeriodFlag ||
         (kiSpatialNum < pSvcParam->iSpatialLayerNum)) {
@@ -306,8 +345,8 @@ EVideoFrameType DecideFrameType (sWelsEncCtx* pEncCtx, const int8_t kiSpatialNum
     } else {
       bSceneChangeFlag = pEncCtx->pVaa->bSceneChangeFlag;
     }
-    if (pEncCtx->pVaa->bIdrPeriodFlag || pEncCtx->bEncCurFrmAsIdrFlag || (!pSvcParam->bEnableLongTermReference
-        && bSceneChangeFlag)) {
+    if (pEncCtx->pVaa->bIdrPeriodFlag || pParamInternal->bEncCurFrmAsIdrFlag || (!pSvcParam->bEnableLongTermReference
+        && bSceneChangeFlag && !bSkipFrameFlag)) {
       iFrameType = videoFrameTypeIDR;
     } else if (pSvcParam->bEnableLongTermReference && (bSceneChangeFlag
                || pEncCtx->pVaa->eSceneChangeIdc == LARGE_CHANGED_SCENE)) {
@@ -328,11 +367,10 @@ EVideoFrameType DecideFrameType (sWelsEncCtx* pEncCtx, const int8_t kiSpatialNum
     } else {
       iFrameType = videoFrameTypeP;
     }
-    if (videoFrameTypeP == iFrameType && pEncCtx->iSkipFrameFlag > 0) {
-      -- pEncCtx->iSkipFrameFlag;
+    if (videoFrameTypeP == iFrameType && bSkipFrameFlag) {
       iFrameType = videoFrameTypeSkip;
     } else if (videoFrameTypeIDR == iFrameType) {
-      pEncCtx->iCodingIndex = 0;
+      pParamInternal->iCodingIndex = 0;
       pEncCtx->bCurFrameMarkedAsSceneLtr   = true;
     }
 
@@ -340,7 +378,7 @@ EVideoFrameType DecideFrameType (sWelsEncCtx* pEncCtx, const int8_t kiSpatialNum
     // perform scene change detection
     if ((!pSvcParam->bEnableSceneChangeDetect) || pEncCtx->pVaa->bIdrPeriodFlag ||
         (kiSpatialNum < pSvcParam->iSpatialLayerNum)
-        || (pEncCtx->iFrameIndex < (VGOP_SIZE << 1))) { // avoid too frequent I frame coding, rc control
+        || (pParamInternal->iFrameIndex < (VGOP_SIZE << 1))) { // avoid too frequent I frame coding, rc control
       bSceneChangeFlag = false;
     } else {
       bSceneChangeFlag = pEncCtx->pVaa->bSceneChangeFlag;
@@ -350,41 +388,50 @@ EVideoFrameType DecideFrameType (sWelsEncCtx* pEncCtx, const int8_t kiSpatialNum
     //bIdrPeriodFlag: RC disable || iSpatialNum != pSvcParam->iSpatialLayerNum
     //pEncCtx->bEncCurFrmAsIdrFlag: 1. first frame should be IDR; 2. idr pause; 3. idr request
     iFrameType = (pEncCtx->pVaa->bIdrPeriodFlag || bSceneChangeFlag
-                  || pEncCtx->bEncCurFrmAsIdrFlag) ? videoFrameTypeIDR : videoFrameTypeP;
+                  || pParamInternal->bEncCurFrmAsIdrFlag) ? videoFrameTypeIDR : videoFrameTypeP;
+    if ( videoFrameTypeIDR == iFrameType ) {
+      WelsLog (& (pEncCtx->sLogCtx), WELS_LOG_DEBUG,
+               "encoding videoFrameTypeIDR due to ( bIdrPeriodFlag %d, bSceneChangeFlag %d, bEncCurFrmAsIdrFlag %d )",
+               pEncCtx->pVaa->bIdrPeriodFlag,
+               bSceneChangeFlag,
+               pParamInternal->bEncCurFrmAsIdrFlag);
+    }
 
-    if (videoFrameTypeP == iFrameType && pEncCtx->iSkipFrameFlag > 0) {  // for frame skip, 1/5/2010
-      -- pEncCtx->iSkipFrameFlag;
+    if (videoFrameTypeP == iFrameType && bSkipFrameFlag) {  // for frame skip, 1/5/2010
       iFrameType = videoFrameTypeSkip;
     } else if (videoFrameTypeIDR == iFrameType) {
-      pEncCtx->iCodingIndex = 0;
+      pParamInternal->iCodingIndex = 0;
     }
   }
   return iFrameType;
 }
 
 /*!
- * \brief	Dump reconstruction for dependency layer
+ * \brief   Dump reconstruction for dependency layer
  */
 
 extern "C" void DumpDependencyRec (SPicture* pCurPicture, const char* kpFileName, const int8_t kiDid, bool bAppend,
-                                   SDqLayer* pDqLayer) {
+                                   SDqLayer* pDqLayer, bool bSimulCastAVC) {
   WelsFileHandle* pDumpRecFile = NULL;
-  int32_t iWrittenSize											= 0;
+  int32_t iWrittenSize = 0;
   const char* openMode = bAppend ? "ab" : "wb";
-  SWelsSPS* pSpsTmp = (kiDid > BASE_DEPENDENCY_ID) ? & (pDqLayer->sLayerInfo.pSubsetSpsP->pSps) :
-                      pDqLayer->sLayerInfo.pSpsP;
+  SWelsSPS* pSpsTmp = NULL;
+  if (bSimulCastAVC || (kiDid == BASE_DEPENDENCY_ID)) {
+    pSpsTmp = pDqLayer->sLayerInfo.pSpsP;
+  } else {
+    pSpsTmp = & (pDqLayer->sLayerInfo.pSubsetSpsP->pSps);
+  }
   bool bFrameCroppingFlag = pSpsTmp->bFrameCroppingFlag;
   SCropOffset* pFrameCrop = &pSpsTmp->sFrameCrop;
 
   if (NULL == pCurPicture || NULL == kpFileName || kiDid >= MAX_DEPENDENCY_LAYER)
     return;
-
-  if (strlen (kpFileName) > 0)	// confirmed_safe_unsafe_usage
+  if (strlen (kpFileName) > 0) // confirmed_safe_unsafe_usage
     pDumpRecFile = WelsFopen (kpFileName, openMode);
   else {
     char sDependencyRecFileName[16] = {0};
-    WelsSnprintf (sDependencyRecFileName, 16, "rec%d.yuv", kiDid);	// confirmed_safe_unsafe_usage
-    pDumpRecFile	= WelsFopen (sDependencyRecFileName, openMode);
+    WelsSnprintf (sDependencyRecFileName, 16, "rec%d.yuv", kiDid); // confirmed_safe_unsafe_usage
+    pDumpRecFile = WelsFopen (sDependencyRecFileName, openMode);
   }
   if (NULL != pDumpRecFile && bAppend)
     WelsFseek (pDumpRecFile, 0, SEEK_END);
@@ -392,13 +439,13 @@ extern "C" void DumpDependencyRec (SPicture* pCurPicture, const char* kpFileName
   if (NULL != pDumpRecFile) {
     int32_t i = 0;
     int32_t j = 0;
-    const int32_t kiStrideY	= pCurPicture->iLineSize[0];
-    const int32_t kiLumaWidth	= bFrameCroppingFlag ? (pCurPicture->iWidthInPixel - ((pFrameCrop->iCropLeft +
-                                pFrameCrop->iCropRight) << 1)) : pCurPicture->iWidthInPixel;
-    const int32_t kiLumaHeight	= bFrameCroppingFlag ? (pCurPicture->iHeightInPixel - ((pFrameCrop->iCropTop +
-                                  pFrameCrop->iCropBottom) << 1)) : pCurPicture->iHeightInPixel;
-    const int32_t kiChromaWidth	= kiLumaWidth >> 1;
-    const int32_t kiChromaHeight	= kiLumaHeight >> 1;
+    const int32_t kiStrideY      = pCurPicture->iLineSize[0];
+    const int32_t kiLumaWidth    = bFrameCroppingFlag ? (pCurPicture->iWidthInPixel - ((pFrameCrop->iCropLeft +
+                                   pFrameCrop->iCropRight) << 1)) : pCurPicture->iWidthInPixel;
+    const int32_t kiLumaHeight   = bFrameCroppingFlag ? (pCurPicture->iHeightInPixel - ((pFrameCrop->iCropTop +
+                                   pFrameCrop->iCropBottom) << 1)) : pCurPicture->iHeightInPixel;
+    const int32_t kiChromaWidth  = kiLumaWidth >> 1;
+    const int32_t kiChromaHeight = kiLumaHeight >> 1;
     uint8_t* pSrc = NULL;
     pSrc = bFrameCroppingFlag ? (pCurPicture->pData[0] + kiStrideY * (pFrameCrop->iCropTop << 1) +
                                  (pFrameCrop->iCropLeft << 1)) : pCurPicture->pData[0];
@@ -406,7 +453,7 @@ extern "C" void DumpDependencyRec (SPicture* pCurPicture, const char* kpFileName
       iWrittenSize = WelsFwrite (pSrc + j * kiStrideY, 1, kiLumaWidth, pDumpRecFile);
       assert (iWrittenSize == kiLumaWidth);
       if (iWrittenSize < kiLumaWidth) {
-        assert (0);	// make no sense for us if writing failed
+        assert (0); // make no sense for us if writing failed
         WelsFclose (pDumpRecFile);
         return;
       }
@@ -419,7 +466,7 @@ extern "C" void DumpDependencyRec (SPicture* pCurPicture, const char* kpFileName
         iWrittenSize = WelsFwrite (pSrc + j * kiStrideUV, 1, kiChromaWidth, pDumpRecFile);
         assert (iWrittenSize == kiChromaWidth);
         if (iWrittenSize < kiChromaWidth) {
-          assert (0);	// make no sense for us if writing failed
+          assert (0); // make no sense for us if writing failed
           WelsFclose (pDumpRecFile);
           return;
         }
@@ -431,27 +478,27 @@ extern "C" void DumpDependencyRec (SPicture* pCurPicture, const char* kpFileName
 }
 
 /*!
- * \brief	Dump the reconstruction pictures
+ * \brief   Dump the reconstruction pictures
  */
 
 void DumpRecFrame (SPicture* pCurPicture, const char* kpFileName, const int8_t kiDid, bool bAppend,
                    SDqLayer* pDqLayer) {
-  WelsFileHandle* pDumpRecFile				= NULL;
+  WelsFileHandle* pDumpRecFile = NULL;
   SWelsSPS* pSpsTmp = (kiDid > BASE_DEPENDENCY_ID) ? & (pDqLayer->sLayerInfo.pSubsetSpsP->pSps) :
                       pDqLayer->sLayerInfo.pSpsP;
   bool bFrameCroppingFlag = pSpsTmp->bFrameCroppingFlag;
   SCropOffset* pFrameCrop = &pSpsTmp->sFrameCrop;
 
-  int32_t iWrittenSize			= 0;
+  int32_t iWrittenSize = 0;
   const char* openMode = bAppend ? "ab" : "wb";
 
   if (NULL == pCurPicture || NULL == kpFileName)
     return;
 
-  if (strlen (kpFileName) > 0) {	// confirmed_safe_unsafe_usage
-    pDumpRecFile	= WelsFopen (kpFileName, openMode);
+  if (strlen (kpFileName) > 0) { // confirmed_safe_unsafe_usage
+    pDumpRecFile = WelsFopen (kpFileName, openMode);
   } else {
-    pDumpRecFile	= WelsFopen ("rec.yuv", openMode);
+    pDumpRecFile = WelsFopen ("rec.yuv", openMode);
   }
   if (NULL != pDumpRecFile && bAppend)
     WelsFseek (pDumpRecFile, 0, SEEK_END);
@@ -459,13 +506,13 @@ void DumpRecFrame (SPicture* pCurPicture, const char* kpFileName, const int8_t k
   if (NULL != pDumpRecFile) {
     int32_t i = 0;
     int32_t j = 0;
-    const int32_t kiStrideY	= pCurPicture->iLineSize[0];
-    const int32_t kiLumaWidth	= bFrameCroppingFlag ? (pCurPicture->iWidthInPixel - ((pFrameCrop->iCropLeft +
-                                pFrameCrop->iCropRight) << 1)) : pCurPicture->iWidthInPixel;
-    const int32_t kiLumaHeight	= bFrameCroppingFlag ? (pCurPicture->iHeightInPixel - ((pFrameCrop->iCropTop +
-                                  pFrameCrop->iCropBottom) << 1)) : pCurPicture->iHeightInPixel;
-    const int32_t kiChromaWidth	= kiLumaWidth >> 1;
-    const int32_t kiChromaHeight	= kiLumaHeight >> 1;
+    const int32_t kiStrideY      = pCurPicture->iLineSize[0];
+    const int32_t kiLumaWidth    = bFrameCroppingFlag ? (pCurPicture->iWidthInPixel - ((pFrameCrop->iCropLeft +
+                                   pFrameCrop->iCropRight) << 1)) : pCurPicture->iWidthInPixel;
+    const int32_t kiLumaHeight   = bFrameCroppingFlag ? (pCurPicture->iHeightInPixel - ((pFrameCrop->iCropTop +
+                                   pFrameCrop->iCropBottom) << 1)) : pCurPicture->iHeightInPixel;
+    const int32_t kiChromaWidth  = kiLumaWidth >> 1;
+    const int32_t kiChromaHeight = kiLumaHeight >> 1;
     uint8_t* pSrc = NULL;
     pSrc = bFrameCroppingFlag ? (pCurPicture->pData[0] + kiStrideY * (pFrameCrop->iCropTop << 1) +
                                  (pFrameCrop->iCropLeft << 1)) : pCurPicture->pData[0];
@@ -473,7 +520,7 @@ void DumpRecFrame (SPicture* pCurPicture, const char* kpFileName, const int8_t k
       iWrittenSize = WelsFwrite (pSrc + j * kiStrideY, 1, kiLumaWidth, pDumpRecFile);
       assert (iWrittenSize == kiLumaWidth);
       if (iWrittenSize < kiLumaWidth) {
-        assert (0);	// make no sense for us if writing failed
+        assert (0); // make no sense for us if writing failed
         WelsFclose (pDumpRecFile);
         return;
       }
@@ -486,7 +533,7 @@ void DumpRecFrame (SPicture* pCurPicture, const char* kpFileName, const int8_t k
         iWrittenSize = WelsFwrite (pSrc + j * kiStrideUV, 1, kiChromaWidth, pDumpRecFile);
         assert (iWrittenSize == kiChromaWidth);
         if (iWrittenSize < kiChromaWidth) {
-          assert (0);	// make no sense for us if writing failed
+          assert (0); // make no sense for us if writing failed
           WelsFclose (pDumpRecFile);
           return;
         }
@@ -500,7 +547,7 @@ void DumpRecFrame (SPicture* pCurPicture, const char* kpFileName, const int8_t k
 
 
 /***********************************************************************************/
-void WelsSetMemZero_c (void* pDst, int32_t iSize) {	// confirmed_safe_unsafe_usage
+void WelsSetMemZero_c (void* pDst, int32_t iSize) { // confirmed_safe_unsafe_usage
   memset (pDst, 0, iSize);
 }
 }
